from simtk.openmm.app import *
from simtk.openmm import *
from simtk.unit import *
from sys import stdout

class IntegratorGenerator:

    def __init__ (self, stepSize, nAtoms, target, tautemp = 0.02):
        self.integrator = CustomIntegrator(stepSize)
        self.df = 3.0 * nAtoms - 6.0
        self.gasconst = 8.314459848e-3
        self.tautemp = tautemp ## Unit is in picosecond
        self.kelvin = float(target)

        ## Variables used in BUSSI thermostat
        self.integrator.addGlobalVariable("df", self.df)
        self.integrator.addGlobalVariable("gasconst", self.gasconst)
        self.integrator.addGlobalVariable("tautemp", self.tautemp)
        self.integrator.addGlobalVariable("kelvin", self.kelvin)
        self.integrator.addGlobalVariable("ke", 0.1)
        self.integrator.addGlobalVariable("temp", 220.0)
        self.integrator.addGlobalVariable("scale", 1.0)

        self.integrator.addGlobalVariable("c", 0.1)
        self.integrator.addGlobalVariable("d", 0.1)
        self.integrator.addGlobalVariable("r", 0.001)
        self.integrator.addGlobalVariable("s", 0.0)
        self.integrator.addPerDofVariable("si", 0.0)

    def setIntegrator (self):
        self.integrator.addUpdateContextState()
        self.integrator.addComputePerDof("v", "v+0.5*dt*f/m")
        self.integrator.addComputePerDof("x", "x+dt*v")
        self.integrator.addConstrainPositions()
        self.integrator.addComputePerDof("v", "v+0.5*dt*f/m")
        self.setBussiThermostat()
        self.integrator.addConstrainVelocities()

    def setBussiThermostat(self):
        self.integrator.addComputeSum("ke", "m*v*v/2.0")
        self.integrator.addComputeGlobal("temp", "2.0*ke/(df *gasconst)")
        self.integrator.addComputeGlobal("temp", "temp + step(0.0000000001-temp) * 0.1")
        self.integrator.addComputeGlobal("c", "exp(-dt/tautemp)")
        self.integrator.addComputeGlobal("d", "(1.0-c)*(kelvin/temp)/df")
        self.integrator.addComputeGlobal("r", "gaussian")
        self.integrator.addComputePerDof("si", "gaussian")
        self.integrator.addComputeSum("s", "si*si")
        self.integrator.addComputeGlobal("s", "s-(gaussian^2+gaussian^2+gaussian^2+gaussian^2+gaussian^2+gaussian^2)")

        self.integrator.addComputeGlobal("scale", "sqrt(c+(s+r*r)*d+2.0*r*sqrt(c*d))")
        self.integrator.addComputeGlobal("scale", "scale-2.0*scale*step(-r-sqrt(c/d))")
        self.integrator.addComputePerDof("v", "scale*v")

    def _printGlobalVariableValues(self):
        totalVariables = self.integrator.getNumGlobalVariables();
        for i in range(totalVariables):
            name = self.integrator.getGlobalVariableName(i)
            value = self.integrator.getGlobalVariable(i)
            print(str(name) + " -> " + str(value))
        print("========================")



    def getIntegrator(self):
        self.setIntegrator()
      	return self.integrator


